=== modified file 'NEWS'
--- NEWS	2008-03-14 01:19:27 +0000
+++ NEWS	2008-03-14 19:44:24 +0000
@@ -60,6 +60,9 @@
      case of running ``bzr missing`` command for two identical branches.
      (Alexander Belchenko)
 
+   * The ``hooks`` command lists installed hooks, to assist in debugging.
+     (Daniel Watkins)
+
   BUGFIXES:
 
     * Disable plink's interactive prompt for password.

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-03-14 01:19:27 +0000
+++ bzrlib/builtins.py	2008-03-14 19:44:24 +0000
@@ -4452,6 +4452,28 @@
             urlutils.unescape_for_display(to_branch.base, 'utf-8'))
 
 
+class cmd_hooks(Command):
+    """Show a branch's currently registered hooks.
+    """
+
+    hidden = True
+    takes_args = ['path?']
+
+    def run(self, path=None):
+        if path is None:
+            path = '.'
+        branch_hooks = Branch.open(path).hooks
+        for hook_type in branch_hooks:
+            hooks = branch_hooks[hook_type]
+            self.outf.write("%s:\n" % (hook_type,))
+            if hooks:
+                for hook in hooks:
+                    self.outf.write("  %s\n" %
+                                    (branch_hooks.get_hook_name(hook),))
+            else:
+                self.outf.write("  <no hooks installed>\n")
+
+
 def _create_prefix(cur_transport):
     needed = [cur_transport]
     # Recurse upwards until we can create a directory successfully

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2007-11-21 15:10:44 +0000
+++ bzrlib/tests/blackbox/__init__.py	2008-03-08 16:35:28 +0000
@@ -64,6 +64,7 @@
                      'bzrlib.tests.blackbox.test_export',
                      'bzrlib.tests.blackbox.test_find_merge_base',
                      'bzrlib.tests.blackbox.test_help',
+                     'bzrlib.tests.blackbox.test_hooks',
                      'bzrlib.tests.blackbox.test_ignore',
                      'bzrlib.tests.blackbox.test_ignored',
                      'bzrlib.tests.blackbox.test_info',

=== added file 'bzrlib/tests/blackbox/test_hooks.py'
--- bzrlib/tests/blackbox/test_hooks.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_hooks.py	2008-03-08 18:25:44 +0000
@@ -0,0 +1,54 @@
+# Copyright (C) 2008 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for commands related to hooks"""
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestHooks(TestCaseWithTransport):
+
+    def _check_hooks_output(self, command_output, hooks):
+        for hook_type in Branch.hooks:
+            s = "\n  ".join(hooks.get(hook_type, ["<no hooks installed>"]))
+            self.assert_("%s:\n  %s" % (hook_type, s) in command_output)
+
+    def test_hooks_with_no_hooks(self):
+        self.make_branch('.')
+        out, err = self.run_bzr('hooks')
+        self.assertEqual(err, "")
+        for hook_type in Branch.hooks:
+            self._check_hooks_output(out, {})
+
+    def test_hooks_with_unnamed_hook(self):
+        self.make_branch('.')
+        def foo(): return
+        Branch.hooks.install_hook('set_rh', foo)
+        out, err = self.run_bzr('hooks')
+        self._check_hooks_output(out, {'set_rh': ["No hook name"]})
+
+    def test_hooks_with_named_hook(self):
+        self.make_branch('.')
+        def foo(): return
+        name = "Foo Bar Hook"
+        Branch.hooks.install_hook('set_rh', foo)
+        Branch.hooks.name_hook(foo, name)
+        out, err = self.run_bzr('hooks')
+        self._check_hooks_output(out, {'set_rh': [name]})
+
+    def test_hooks_no_branch(self):
+        self.run_bzr('hooks', retcode=3)

=== modified file 'doc/en/user-guide/hooks.txt'
--- doc/en/user-guide/hooks.txt	2007-12-17 02:00:45 +0000
+++ doc/en/user-guide/hooks.txt	2008-03-11 00:45:46 +0000
@@ -49,3 +49,10 @@
 
 For a complete list of hooks and their parameters, see `Hooks
 <../user-reference/bzr_man.html#hooks>`_ in the User Reference.
+
+Debugging hooks
+---------------
+
+To get a list of installed hooks, use the hidden ``hooks`` command::
+
+    bzr hooks

